/*
 * Virtual Drum Machine - Rhythm Compiler runtime library
 * This project is in the public domain.
 *---
 * The card stuff
 */

#include "card.h"

#include <unistd.h>
#include <fcntl.h>
#include <sys/soundcard.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>

/* return 0 if ok, -1 if not */
int init_card(CARD *c)
{
  int fragsize = 13;
  int numfrags = 5;
  int fragmentsize=(numfrags<<16) | fragsize;
  int nb_channels = 1;
  int format = AFMT_S16_LE;
  int freq=c->frequency;

  if (c->mode == CARD_NO_SOUND)
    return 0;

  if (c->mode == CARD_STEREO)
    nb_channels = 2;

  c->fd = open(c->device, O_WRONLY);
  if (c->fd == -1) {
    perror(c->device);
    return -1;
  }

  /* oss doc says to do this ioctl first */
  if(ioctl(c->fd, SNDCTL_DSP_SETFRAGMENT, &fragmentsize)<0) {
    perror(c->device);
    return -1;
  }
  /* we don't check it, it's not a big issue in this application */

  /* order conforming to the oss specs */
  if (ioctl(c->fd, SNDCTL_DSP_SETFMT, &format) == -1 ||
      ioctl(c->fd, SNDCTL_DSP_CHANNELS, &nb_channels)==-1 ||
      ioctl(c->fd, SNDCTL_DSP_SPEED, &freq)==-1) {
    perror(c->device);
    return -1;
  }

  if (format != AFMT_S16_LE ||
      nb_channels != (c->mode == CARD_STEREO ? 2 : 1) ||
      c->frequency != freq) {
    fprintf(stderr, "%s: error with one SNDCTL_DSP_*, sorry\n", c->device);
    return -1;
  }

  return 0;
}

void close_card(CARD *c)
{
  if (c->mode == CARD_NO_SOUND)
    return;

  close(c->fd);
}

/* size is in bytes */
void card_play(CARD *c, void *buf, int size)
{
  if (c->mode == CARD_NO_SOUND)
    return;

  if (write(c->fd, buf, size) != size) {
    perror(c->device);
    exit(1);
  }
}
