/*
 * Virtual Drum Machine - Rhythm Compiler runtime library
 * This project is in the public domain.
 *---
 * The sound stuff
 */

#include "sound.h"

#include <stdio.h>

#include "mixer.h"
#include "card.h"
#include "sample.h"

/* return 0 if ok, -1 if error (with some output on stderr) */
int init_sound(SOUND *s)
{
  if (init_mixer(&s->mixer)==-1) {
    fprintf(stderr, "init_sound : error initing the mixer stuff\n");
    return -1;
  }

  if (init_card(&s->card)==-1) {
    fprintf(stderr, "init_sound : error initing the soundcard stuff\n");
    return -1;
  }
  
  if (init_wave(&s->wav)==-1) {
    fprintf(stderr, "init_sound : error initing the WAV stuff\n");
    return -1;
  }

  return 0;
}

void play_some_sound(SOUND *s, int dur)
{
  /*
    tempo notes / minutes
    1/dur note
    44100 samples / sec

    tempo / dur  -  1
          x      -  1 / 60

    y = tempo / 60 / dur * 44100
   */
  /* this is bad, we should handle floating point values, we have bad precision here */
  int nbsamples = (double)s->mixer.frequency * 60. / ((double)dur * (double)s->mixer.tempo);

  if (s->mixer.mode == MIXER_STEREO || s->mixer.mode == MIXER_SEPARATED)
    nbsamples *= 2;

  mix_some(&s->mixer, nbsamples*2);

  if (s->card.mode != CARD_NO_SOUND)
    card_play(&s->card, s->mixer.buffer_out, nbsamples*2);
  if (s->wav.mode != WAV_DO_NOTHING)
    wave_play(&s->wav, s->mixer.buffer_out, nbsamples*2);
}

void finalize_sound(SOUND *s)
{
  while(s->mixer.list) {
    mix_some(&s->mixer, 4096);

    if (s->card.mode != CARD_NO_SOUND)
      card_play(&s->card, s->mixer.buffer_out, 4096);
    if (s->wav.mode != WAV_DO_NOTHING)
      wave_play(&s->wav, s->mixer.buffer_out, 4096);
  }
}

void put_note(SOUND *s, SAMPLE *sp, double volume, double panning)
{
  if (panning < -1 || panning > 1) {
    fprintf(stderr, "warning panning = %f, invalid value, ", panning);
    if (panning < -1) panning = -1;
    if (panning > 1) panning = 1;
    fprintf(stderr, "set to correct value %d\n", (int)panning);
  }

  mixer_add_channel(&s->mixer, sp, volume, panning);
}

void close_sound(SOUND *s)
{
  close_card(&s->card);
  close_wave(&s->wav);
  /* must come last */
  close_mixer(&s->mixer);
}
