/*
 * Virtual Drum Machine - Rhythm Compiler program
 * This project is in the public domain.
 *---
 * The error definition file
 */

#include "error.h"

#include <stdio.h>
#include <stdlib.h>

void usage(char *p)
{
  fprintf(stderr, "rc (version %s) - the rhythm compiler\n", VERSION);
  fprintf(stderr, "usage: %s <options> <input file>\n", p);
  fprintf(stderr, "options:\n");
  fprintf(stderr, "    -h | --help    : to print this help screen\n");
  fprintf(stderr, "    -v | --version : to print the version of rc\n");
  fprintf(stderr, "    -c         : only translate into c program\n");
  fprintf(stderr, "    -o <file>  : set output file (default a.out)\n");
  fprintf(stderr, "    -cf <file> : to use the specified configuration\n");
  fprintf(stderr, "        if not set, we will first seach rc.conf in current dir, then in $HOME,\n");
  fprintf(stderr, "        then in %s (which is where rc is installed)\n", INSTALL_DIR);
  fprintf(stderr, "    --config-file <file> : same as -cf\n");
}

char *error_type[] = {
  "no error",
  "environnement init",
  "compiler init",
  "rhythm init",
  "compile"
};

char *error_code[] = {
  "no error",
  "file not found",
  "note expected",
  "bad instrument name",
  "getline error",
  "can't create file",
  "input file already defined",
  "bad option name",
  "output file already defined",
  "file name expected",
  "'c only' already defined",
  "impossible to create temp name",
  "fork error",
  "gcc error",
  "exec error",
  "input file required",
  "config file not found (searched rc.conf in local directory, then $HOME/rc.conf then " INSTALL_DIR "/rc.conf)\ntry the --help option",
  "no config file name",
  "bad config file name",
  "env INCLUDE_DIR not defined",
  "env LIBRARY_DIR not defined",
  "error writing file",
  "parse error",
  "parse error: the first note of your rhythm MUST be '*'",
  "parse error: poly-note (one inside ')' and '(' ) can't be broken by normal C code lines"
};

void error(int type, int ret_code, int n, char **v)
{
  fprintf(stderr, "%s: %s\n", error_type[type], error_code[-ret_code]);
  exit(type);
}
