/*
 * Virtual Drum Machine - Rhythm Compiler program
 * This project is in the public domain.
 *---
 * The main file
 */

#include "error.h"
#include "compiler.h"
#include "rhythm.h"
#include "environment.h"

#include <stdio.h>

int main(int n, char **v)
{
  ENVIRONMENT environment;
  COMPILER compiler;
  RHYTHM rhythm;
  int ret_code;

  if ((ret_code=init_environment(&environment, n, v))<0)
    error(ENVIRONMENT_INIT, ret_code, n, v);

  if (ret_code == 1) {
    usage(v[0]);
    return 0;
  }

  if (ret_code == 2) {
    fprintf(stdout, "rc version %s\n", VERSION);
    return 0;
  }

  /* init_compiler must come after init_environment */
  if ((ret_code=init_compiler(&compiler, n, v, &environment))<0)
    error(COMPILER_INIT, ret_code, n, v);

  if ((ret_code=init_rhythm(&rhythm, compiler.input_file, &environment))<0)
    error(RHYTHM_INIT, ret_code, n, v);

  if ((ret_code=compile(&compiler, &rhythm, &environment))<0)
    error(COMPILE, ret_code, n, v);

  return 0;
}
