/*
 * Virtual Drum Machine - Rhythm Compiler program
 * This project is in the public domain.
 *---
 * The rhythm definition file
 */

#include "rhythm.h"

#ifndef _GNU_SOURCE
#define _GNU_SOURCE	/* I use getline, so I need this #define */
#endif
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>

#include "environment.h"
#include "memory.h"
#include "error.h"

void append_instrument(NOTE *n, INSTRUMENT *inst)
{
  if (n->nb_elem==n->size) {
    n->instruments=regetmem(n->instruments, (n->size+8)*sizeof(INSTRUMENT));
    n->size+=8;
  }

  memcpy(&n->instruments[n->nb_elem], inst, sizeof(INSTRUMENT));

  n->nb_elem++;
}

void append_note(RHYTHM *r, NOTE *n)
{
  if (r->nb_elem==r->size) {
    r->notes=regetmem(r->notes, (r->size+512)*sizeof(NOTE));
    r->size+=512;
  }

  memcpy(&r->notes[r->nb_elem], n, sizeof(NOTE));

  r->nb_elem++;
}

void new_note(RHYTHM *r, int l)
{
  if (r->nsize == r->nmaxsize)
    r->pnotes = regetmem(r->pnotes, (r->nmaxsize+=32) * sizeof(POLYNOTE));

  memset(&r->pnotes[r->nsize], 0, sizeof(POLYNOTE));

  r->pnotes[r->nsize].line_in_file = l;

  r->nsize++;
}

void new_poly(RHYTHM *r, int l)
{
  POLYNOTE *cur = &r->pnotes[r->nsize-1];

  if (cur->size == cur->maxsize)
    cur->data = regetmem(cur->data, (cur->maxsize+=32) * sizeof(POLY));

  memset(&cur->data[cur->size], 0, sizeof(POLY));

  cur->data[cur->size].line_in_file = l;

  cur->size++;
}

void new_sub_note(RHYTHM *r, int l)
{
  POLY *cur = &r->pnotes[r->nsize-1].data[r->pnotes[r->nsize-1].size-1];

  if (cur->size == cur->maxsize)
    cur->data = regetmem(cur->data, (cur->maxsize+=32) * sizeof(NOTE));

  memset(&cur->data[cur->size], 0, sizeof(NOTE));

  cur->data[cur->size].line_in_file = l;

  cur->size++;
}

void new_inst(RHYTHM *r, int l, char *n)
{
  POLY *cur = &r->pnotes[r->nsize-1].data[r->pnotes[r->nsize-1].size-1];
  NOTE *nt = &cur->data[cur->size-1];
  INSTRUMENT inst;

  inst.name = n;

  append_instrument(nt, &inst);
}

/* return 0 if ok, -<error type> if not */
static int handle_token(int tok, char *name, RHYTHM *r, int current_line, ENVIRONMENT *env)
{
  /* weird indent mode was on here */

  switch (r->state) {
  case INIT :
  switch (tok) {
    case TNOTE :
      r->state = STARTED;
      new_note(r, current_line);
      new_poly(r, current_line);
      new_sub_note(r, current_line);
      return 0;

    default :
      return -PARSER_BAD_INIT_NOTE;
  }

  case STARTED :
  switch (tok) {
    case TSUBNOTE :
      r->state = FLAT;
      new_sub_note(r, current_line);
      return 0;

    case TINST :
      r->state = FLAT;
      new_inst(r, current_line, name);
      return 0;

    case TNOTE :
      r->state = STARTED;
      new_note(r, current_line);
      new_poly(r, current_line);
      new_sub_note(r, current_line);
      return 0;

    case TPARL :
      r->state = RECURSE;
      new_poly(r, current_line);
      new_sub_note(r, current_line);
      return 0;

    default :
      return -PARSE_ERROR;
  }

  case FLAT :
  switch (tok) {
    case TINST :
      r->state = FLAT;
      new_inst(r, current_line, name);
      return 0;

    case TSUBNOTE :
      r->state = FLAT;
      new_sub_note(r, current_line);
      return 0;

    case TNOTE :
      r->state = STARTED;
      new_note(r, current_line);
      new_poly(r, current_line);
      new_sub_note(r, current_line);
      return 0;

    default :
      return -PARSE_ERROR;
  }
    
  case RECURSE :
  switch (tok) {
    case TINST :
      r->state = RECURSE;
      new_inst(r, current_line, name);
      return 0;

    case TSUBNOTE :
      r->state = RECURSE;
      new_sub_note(r, current_line);
      return 0;

    case TPARR :
      r->state = RECURSE_END;
      return 0;

    default :
      return -PARSE_ERROR;
  }

  case RECURSE_END :
  switch (tok) {
    case TNOTE :
      r->state = STARTED;
      new_note(r, current_line);
      new_poly(r, current_line);
      new_sub_note(r, current_line);
      return 0;

    case TPARL :
      r->state = RECURSE;
      new_poly(r, current_line);
      new_sub_note(r, current_line);
      return 0;

    default :
      return -PARSE_ERROR;
  }}

  fprintf(stderr, "impossible error, contact sed@free.fr\n");
  abort();
}

/* return 0 if ok, <0 if error */
int parse_rhythm_line(RHYTHM *rhythm, char *line, int size, int current_line, ENVIRONMENT *env)
{
  char *name;
  int name_size;
  int ret;

  while (size) {
#if 0
    memset(&new_note, 0, sizeof(NOTE));
    new_note.line_in_file = current_line;
#endif

    if (*line == '*') {
      size--; line++;
      if ((ret=handle_token(TNOTE, 0, rhythm, current_line, env)) < 0) return ret;
      goto next_tok;
    }

    if (*line == '.') {
      size--; line++;
      if ((ret=handle_token(TSUBNOTE, 0, rhythm, current_line, env)) < 0) return ret;
      goto next_tok;
    }

    if (*line == '(') {
      size--; line++;
      if ((ret=handle_token(TPARL, 0, rhythm, current_line, env)) < 0) return ret;
      goto next_tok;
    }

    if (*line == ')') {
      size--; line++;
      if ((ret=handle_token(TPARR, 0, rhythm, current_line, env)) < 0) return ret;
      goto next_tok;
    }

    if (!isalpha(*line))
      return -BAD_INSTRUMENT_NAME;

    /* get the instrument's name size */
    name_size=0;
    while (size-name_size && isalnum(*(line+name_size)))
      name_size++;
    /* copy name */
    name = getmem(name_size+1);
    memcpy(name, line, name_size);
    name[name_size]=0;

    line+=name_size;
    size-=name_size;

    if ((ret=handle_token(TINST, name, rhythm, current_line, env)) < 0) return ret;

next_tok:
    while (size && isspace(*line))
      line++, size--;
  }

  /* here we must re-create the array of notes for the compiler */
  /* we must put polynotes the one after the other */
  /* we must mix together the != poly of each polynotes */

  return 0;
}

unsigned long pgcd(unsigned long a, unsigned long b)
{
  /* Euclid was a crazy guy */
  unsigned long q, r, r1;

  if (b == a) return a;

  if (b > a) { unsigned long r = b; b = a; a = r; }

  /* those two operations can be done with only one on some architectures... */
  q = a / b;
  r = a - q * b;
  if (r == 0) return b;

  while (r) {
    r1 = r;
    a = b;
    b = q;
    q = a / b;
    r = a - q * b;
  }

  return r1;
}

unsigned long ppcm(unsigned long a, unsigned long b)
{
  unsigned long c = pgcd(a, b);
  unsigned long ret = a / c;

  return ret * b;
}

void translate_poly_note(RHYTHM *r, POLYNOTE *n)
{
  unsigned long nb_sub_rhythm = n->size;
  unsigned long i;
  unsigned long cm;
  unsigned long int cur_sub;
  POLY *p = n->data;
  NOTE nn;

  /* if we have more than 1 sub-rhythm, we must decrement by 1, because the program will have inserted a void sub-rhythm */
  if (nb_sub_rhythm > 1) {
    nb_sub_rhythm--;
    p++;
  }

  cm = p[0].size;

  for (i=1; i<nb_sub_rhythm; i++)
    cm = ppcm(cm, p[i].size);

  /* we differentiate both case for we need good numbering of lines */
  if (nb_sub_rhythm == 1) {
    for (cur_sub = 0; cur_sub < cm; cur_sub++) {
      p[0].data[cur_sub].duration = cm;

      append_note(r, &p[0].data[cur_sub]);
    }

    return;
  }

  /* big for of death - if I read it in 2 years, I won't get the point anymore :-) */
  /* to be simple : we run through each little chunk of note, search all sub-rhythm,
   * if a note of the sub-rhythm start at the little chunk start, we put it.
   * We claim that all the notes appears on the same line, because the compiler uses this
   * information to mix user code and generated code. No user can come in the middle
   * of a poly-rhythm portion, but this one may come on several lines, but the output
   * may mix info from != lines, so we say all is on the same line. Ok, not clear at all...
   */
  for (cur_sub = 0; cur_sub < cm; cur_sub++) {
    memset(&nn, 0, sizeof(NOTE));

    nn.duration = cm;
    nn.line_in_file = p[-1].line_in_file;	/* weird, huh ? -1 is correct */

    /* we must put the instrument of cur_sub, by running through all the sub_rhythm,
     * checking for por cur_sub, putting all instruments of this pos
     */
    for (i=0; i<nb_sub_rhythm; i++) {
      int j;
      unsigned long tc = cur_sub * p[i].size / cm;
      if (tc * cm != cur_sub * p[i].size) continue;

      for (j=0; j<p[i].data[tc].nb_elem; j++)
        append_instrument(&nn, &p[i].data[tc].instruments[j]);
    }

    append_note(r, &nn);
  }
}

/* return 0 if ok, <0 if error */
int init_rhythm(RHYTHM *rhythm, char *file, ENVIRONMENT *env)
{
  FILE *f=fopen(file, "r");
  char *line=0;
  char *cur;
  int current_line=0;
  NOTE *last_note;
  int i;

  if (!f)
    return -FILE_NOT_FOUND;

  memset(rhythm, 0, sizeof(RHYTHM));

  while (!feof(f)) {
    ssize_t size;
    int ret;
    size_t n=0;
    current_line++;

    /* let's read a line from the file */
    if ((size=getline(&line, &n, f))==-1 && !feof(f))
      return -GETLINE_ERROR;
    /* maybe end of file, so break */
    if (size==-1)
      break;
    /* useless check ? */
    if (size==0) {
      fprintf(stderr, "impossible error size=0, contact sed@free.fr\n");
      exit(1);
    }

    /* forget trailing '\n' */
    size--;

    /* let's see if it is a rhythm line or not */
    cur=line;
    while (size && isspace(*cur))
      cur++, size--;
    if (!size || !isnote(*cur)) {
      if (rhythm->state == RECURSE) return -NO_C_LINE_INSIDE_POLY;
      continue;
    }

    /* it is a rhythm line, let's parse it */
    if ((ret=parse_rhythm_line(rhythm, cur, size, current_line, env))<0)
      return ret;
  }

  free(line);
  fclose(f);

  /* we now must flatten our parsed stuff */
  for (i=0; i < rhythm->nsize; i++)
    translate_poly_note(rhythm, &rhythm->pnotes[i]);

  /* let's insert a final note */
  last_note=getmem(sizeof(NOTE));
  memset(last_note, 0, sizeof(NOTE));
  last_note->duration=1;
  last_note->line_in_file=-1;
  append_note(rhythm, last_note);

#if 0
  /* now that we have all our notes, we can calculate their durations */
  /* the for of death ! you will never understand it, believe me, I dont even do it myself :) */
  for (note=0; note < rhythm->nb_elem-1;) {
    int nb_sub_notes=1;
    int nnote=note+1;
    while (rhythm->notes[nnote].duration==0)
      nnote++, nb_sub_notes++;
    while (note != nnote)
      rhythm->notes[note++].duration = nb_sub_notes;
  }
#endif

  /* ok, it's done ! */
  return 0;
}
