/*
 * Virtual Drum Machine - Rhythm Compiler program
 * This project is in the public domain.
 *---
 * The rhythm definition include file
 */

#ifndef _RHYTHM_H_
#define _RHYTHM_H_

typedef struct {
  char *name;		/* the name of the instrument (read the TECHNICAL file to understand) */
//  int volume;		/* the volume of the instrument : 0..100 */
} INSTRUMENT;

typedef struct {
  INSTRUMENT *instruments;	/* the instruments played at that beat time */
  int nb_elem;			/* the number of elements in the above array */
  int size;			/* the size in the memory of the array (in INSTRUMENT) */
  int duration;			/* the duration of the note (in the inverse of number of black, a croche is 0.5 black) */
  int line_in_file;		/* the line in the input file where the instrument resides */
} NOTE;

/* token type of parser */
#define TNOTE		1
#define TSUBNOTE	2
#define TPARL		3
#define TPARR		4
#define TINST		5

/* states of the parser */
#define	INIT		0	/* the initial state */
#define STARTED		1	/* we read the first note */
#define FLAT		2	/* a sub-note has been read (no '(') */
#define RECURSE		3	/* we read a '(' */
#define RECURSE_END	4	/* we read a ')' matching a '(' */

/*
   INIT -> STARTED if we read "*"
   INIT -> error if anything else

   STARTED -> FLAT if we read an instrument or "."
   STARTED -> STARTED if we read "*"
   STARTED -> RECURSE if we read "("
   STARTED -> error if anything else

   FLAT -> FLAT if we read an instrument or "."
   FLAT -> STARTED if we read "*"
   FLAT -> error if anything else

   RECURSE -> RECURSE if we read an instrument or "."
   RECURSE -> RECURSE_END if we read ")"
   RECURSE -> error if anything else

   RECURSE_END -> STARTED if we read "*"
   RECURSE_END -> RECURSE if we read "("
   RECURSE_END -> error if anything else
*/

typedef struct {
  NOTE *data;
  int size;
  int maxsize;
  int line_in_file;	/* the first line where the poly appears */
} POLY;

typedef struct {
  POLY *data;
  int size;
  int maxsize;
  int line_in_file;	/* the first line where the polynote appears */
} POLYNOTE;

typedef struct {
  /* what will use to compiler */
  NOTE *notes;		/* the array of notes of the rhythm */
  int nb_elem;		/* the size of the rhythm */
  int size;		/* the size of the previous array (in NOTE) */
  /* what uses the parser */
  POLYNOTE *pnotes;
  int nsize;
  int nmaxsize;
  int state;		/* the state of the parser */
} RHYTHM;

#include "environment.h"

int init_rhythm(RHYTHM *, char *, ENVIRONMENT *);

#define isnote(x)	((x)=='*' || (x)=='.')

#endif /* _RHYTHM_H_ */
